using System;
using System.Configuration;
using System.Windows.Forms;

using System.Data;
using System.Data.SqlClient;

public class TransactionDataAdapterForm : System.Windows.Forms.Form
{
	private const String ORDERS_TABLE	= "Orders";

	private DataTable dt;
	private SqlDataAdapter da;
	private SqlCommandBuilder cb;

	private System.Windows.Forms.Button updateButton;
	private System.Windows.Forms.DataGrid dataGrid;
	private System.Windows.Forms.Button refreshButton;

	/// <summary>
	/// Required designer variable.
	/// </summary>
	private System.ComponentModel.Container components = null;

	public TransactionDataAdapterForm()
	{
		//
		// Required for Windows Form Designer support
		//
		InitializeComponent();

		//
		// TODO: Add any constructor code after InitializeComponent call
		//
	}

	/// <summary>
	/// Clean up any resources being used.
	/// </summary>
	protected override void Dispose( bool disposing )
	{
		if( disposing )
		{
			if(components != null)
			{
				components.Dispose();
			}
		}
		base.Dispose( disposing );
	}

	#region Windows Form Designer generated code
	/// <summary>
	/// Required method for Designer support - do not modify
	/// the contents of this method with the code editor.
	/// </summary>
	private void InitializeComponent()
	{
		this.dataGrid = new System.Windows.Forms.DataGrid();
		this.updateButton = new System.Windows.Forms.Button();
		this.refreshButton = new System.Windows.Forms.Button();
		((System.ComponentModel.ISupportInitialize)(this.dataGrid)).BeginInit();
		this.SuspendLayout();
		// 
		// dataGrid
		// 
		this.dataGrid.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
			| System.Windows.Forms.AnchorStyles.Left) 
			| System.Windows.Forms.AnchorStyles.Right);
		this.dataGrid.DataMember = "";
		this.dataGrid.HeaderForeColor = System.Drawing.SystemColors.ControlText;
		this.dataGrid.Location = new System.Drawing.Point(8, 40);
		this.dataGrid.Name = "dataGrid";
		this.dataGrid.Size = new System.Drawing.Size(476, 216);
		this.dataGrid.TabIndex = 0;
		// 
		// updateButton
		// 
		this.updateButton.Location = new System.Drawing.Point(408, 8);
		this.updateButton.Name = "updateButton";
		this.updateButton.TabIndex = 2;
		this.updateButton.Text = "Update";
		this.updateButton.Click += new System.EventHandler(this.updateButton_Click);
		// 
		// refreshButton
		// 
		this.refreshButton.Location = new System.Drawing.Point(328, 8);
		this.refreshButton.Name = "refreshButton";
		this.refreshButton.TabIndex = 1;
		this.refreshButton.Text = "Refresh";
		this.refreshButton.Click += new System.EventHandler(this.refreshButton_Click);
		// 
		// TransactionDataAdapterForm
		// 
		this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
		this.ClientSize = new System.Drawing.Size(492, 266);
		this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		this.refreshButton,
																		this.updateButton,
																		this.dataGrid});
		this.Name = "TransactionDataAdapterForm";
		this.Text = "6.05 TransactionDataAdapterForm";
		this.Load += new System.EventHandler(this.TransactionDataAdapterForm_Load);
		((System.ComponentModel.ISupportInitialize)(this.dataGrid)).EndInit();
		this.ResumeLayout(false);

	}
	#endregion

	private void TransactionDataAdapterForm_Load(object sender, System.EventArgs e)
	{
		String sqlText = "SELECT * FROM Orders";

		// fill the Orders table for editing
		da = new SqlDataAdapter(sqlText, ConfigurationSettings.AppSettings["Sql_ConnectString"]);
		// stop updating when an error is encountered for roll back
		da.ContinueUpdateOnError = false;
		// create CommandBuilder and generate updating logic
		cb = new SqlCommandBuilder(da);
		cb.GetDeleteCommand();
		cb.GetInsertCommand();
		cb.GetUpdateCommand();
		// create table and fill with orders schema and data
		dt = new DataTable(ORDERS_TABLE);
		da.FillSchema(dt, SchemaType.Source);
		da.Fill(dt);

		// bind the default view of the table to the grid
		dataGrid.DataSource = dt.DefaultView;
	}

	private void updateButton_Click(object sender, System.EventArgs e)
	{
		// create and open the connection
		SqlConnection conn = new SqlConnection(ConfigurationSettings.AppSettings["Sql_ConnectString"]);
		conn.Open();

		// create and start the transaction
		SqlTransaction tran = null;
		tran = conn.BeginTransaction();

		// associate CommandBuilder generated update commands
		// with the transaction
		da.SelectCommand.Transaction = tran;

		// update the data source
		try
		{
			// submit the changes
			da.Update(dt);

			// success - commit
			tran.Commit();
		}
		catch (Exception ex)
		{
			// exception - roll back
			tran.Rollback();

			MessageBox.Show(ex.Message + Environment.NewLine + "Transaction rolled back.");
		}
		finally
		{
			conn.Close();
		}
	}

	private void refreshButton_Click(object sender, System.EventArgs e)
	{
		// refresh the orders data
		dt.Clear();
		da.Fill(dt);
	}
}